import argparse

import oss2
import os
import json
import zipfile
import shutil

from . import common
from .lib_repo import LibRepo, LibData

from .common import get_bucket, zip_dir, unzip_file, upload_file, download_file, OSS_BASE_PATH, get_lib_zip_file_name, get_lib_zip_file_key, get_platform_name, available_build_names, available_arches

import logging
logger = logging.getLogger(__name__)

DEPENDENCY_JSON_FILE_NAME = 'dependencies.json'


def get_default_dep_path():
    return os.path.join(os.getcwd(), DEPENDENCY_JSON_FILE_NAME)


def get_dep(dep_path):
    with open(dep_path, 'r') as f:
        return json.load(f)


def download_lib(lib_data: LibData, to_path: str):
    file_key = lib_data.file_key()
    _, zip_name = os.path.split(file_key)
    local_path = lib_data.gen_path()
    local_path = os.path.join(to_path, local_path)
    if os.path.exists(local_path):
        print(f'[warning] {local_path} already exist.')
        return

    zip_file = os.path.join(local_path, zip_name)

    os.makedirs(local_path)
    download_file(file_key, zip_file)

    unzip_file(zip_file, local_path)
    os.remove(zip_file)


def parse_args():
    description = "Download dependencies lib."
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('--platform', help='Platform', choices=common.available_platforms())
    parser.add_argument('--arch', help='Lib architecture', choices=common.all_available_arches())
    parser.add_argument('--build', help='Lib build type', choices=common.available_build_names())
    parser.add_argument('--dep', help='The dependencies json file in which the libs will be download from lib repo')
    parser.add_argument('--dest', help='Where download the libs to')

    sub_parsers = parser.add_subparsers(dest='sub_command')
    list_parser = sub_parsers.add_parser('list')

    return parser.parse_args()


def try_find_compatible_libs(repo: LibRepo, name: str, ver: str, platform: str, arch=None, build=None):
    libs = []
    if not arch and not build:
        for b in available_build_names():
            for p in available_arches(platform):
                lib = LibData(name=name, version=ver, platform=platform, arch=p, build=b)
                if repo.exist(lib):
                    libs.append(lib)
    if arch and not build:
        for b in available_build_names():
            lib = LibData(name=name, version=ver, platform=platform, arch=arch, build=b)
            if repo.exist(lib):
                libs.append(lib)
    if not arch and build:
        for p in available_arches(platform):
            lib = LibData(name=name, version=ver, platform=platform, arch=p, build=build)
            if repo.exist(lib):
                libs.append(lib)

    return libs


def list_command(args):
    print(str(LibRepo().pull_repo()))


def lib_get():
    args = parse_args()

    if args.sub_command == 'list':
        return list_command(args)

    def strip(s):
        return s.strip() if s else None

    if not args.dep:
        args.dep = get_default_dep_path()
    if not args.dest:
        args.dest, _ = os.path.split(args.dep)
    if not args.platform:
        args.platform = get_platform_name()

    deps = get_dep(args.dep)
    libs = []
    repo = LibRepo().pull_repo()

    platform = strip(args.platform)
    arch = strip(args.arch)
    build = strip(args.build)
    for name, ver in deps.items():
        name = strip(name)
        ver = strip(ver)
        lib = LibData(name=name, version=ver, platform=platform, arch=arch, build=build)
        if repo.exist(lib):
            libs.append(lib)
            continue

        compatible_libs = try_find_compatible_libs(repo, name, ver, platform, arch, build)
        if compatible_libs:
            libs.extend(compatible_libs)
        else:
            print(f'[error] can not find lib from repo. name: {name}, version: {ver}, platform: {platform}, architecture: {arch}, build: {build}')

    for lib in libs:
        download_lib(lib, args.dest)


if __name__ == '__main__':
    lib_get()



import json
import os
import argparse
import shutil
from dataclasses import dataclass, asdict
from typing import Optional

from .common import is_legal_local_lib_dir
from . import common
from dacite import from_dict
from tabulate import tabulate

import logging
logger = logging.getLogger(__name__)


@dataclass
class LibData:
    name: str
    version: Optional[str]
    platform: Optional[str]
    arch: Optional[str]
    build: Optional[str]

    @classmethod
    def from_dict(cls, data) -> "LibData":
        return from_dict(LibData, data)

    def to_dict(self):
        return asdict(self)

    def file_key(self):
        if not self.name:
            raise Exception(f'invalid lib data: {self}')
        file_key = os.path.join(common.OSS_BASE_PATH, self.gen_path())
        file_key = os.path.join(file_key, self.get_zip_file_name())

        file_key = file_key.replace('\\', '/')

        return file_key

    def gen_path(self):
        p = self.name
        if self.version:
            p = os.path.join(p, self.version)
        if self.platform:
            p = os.path.join(p, self.platform)
        if self.arch:
            p = os.path.join(p, self.arch)
        if self.build:
            p = os.path.join(p, self.build)

        return p

    def get_zip_file_name(self):
        return self.name + "_lib.zip"

    def __str__(self):
        return f'{self.name}\t{self.version}\t{self.platform}\t{self.arch}\t{self.build}'


class LibRepo:
    def __init__(self):
        self.libs: Optional[list[LibData]] = None
        self.__repo_file_key = os.path.join(common.OSS_BASE_PATH, 'repo.json')
        self.__repo_file_key = self.__repo_file_key.replace('\\', '/')
    def pull_repo(self) -> "LibRepo":
        content = common.get_file_obj_2_str(self.__repo_file_key)
        if not content:
            return self

        libs_data = json.loads(content)
        self.libs = [LibData.from_dict(lib) for lib in libs_data]

        return self

    def push_repo(self) -> "LibRepo":
        if not self.libs:
            return self

        libs = [lib.to_dict() for lib in self.libs]
        common.upload_file_obj(self.__repo_file_key, json.dumps(libs))

        return self

    def add_new_lib(self, lib: LibData):
        if not self.libs:
            self.libs = []
        if self.exist(lib):
            raise Exception('already exist')
        self.libs.append(lib)

    def exist(self, lib: LibData):
        if not self.libs:
            return False

        return any(lb == lib for lb in self.libs)

    def __str__(self):
        def sort_key(lib: LibData):
            return f'{lib.name}-{lib.version}-{lib.platform}-{lib.arch}-{lib.build}'
        sorted_libs = sorted(self.libs, key=sort_key)

        table_headers = ['Name', 'Version', 'Platform', 'Architecture', 'Build']
        table_rows = []
        for l in sorted_libs:
            cells = [l.name, l.version, l.platform, l.arch or '', l.build or '']
            table_rows.append(cells)

        return str(tabulate(table_rows, headers=table_headers))


def update_lib(local_path: str, lib: LibData):
    if not os.path.exists(local_path):
        logger.error(f'Lib path not exit: {local_path}')
        raise Exception('lib path not exist!')

    zip_file_name = lib.get_zip_file_name()
    zip_file_path = os.path.join(local_path, zip_file_name)
    common.zip_dir(local_path, zip_file_path)

    file_key = lib.file_key()
    common.upload_file(zip_file_path, file_key)
    os.remove(zip_file_path)

    repo = LibRepo()
    repo.pull_repo()
    if not repo.exist(lib):
        repo.add_new_lib(lib)
        repo.push_repo()


def parse_args():
    description = "Upload your lib to cloud"
    parser = argparse.ArgumentParser(description=description)
    sub_parsers = parser.add_subparsers(dest='sub_command')
    add_parser = sub_parsers.add_parser('add')
    add_parser.add_argument('--path', help='The library folder you want to upload.')
    add_parser.add_argument('--name', help='The library name.')
    add_parser.add_argument('--version', help='The library version.')
    add_parser.add_argument('--platform', help='Platform the library running.', choices=common.available_platforms())
    add_parser.add_argument('--arch', help='The library architecture.', choices=common.all_available_arches())
    add_parser.add_argument('--build', help='The library build type.', choices=common.available_build_names())

    remove_parser = sub_parsers.add_parser('remove')
    remove_parser.add_argument('--name')

    return parser.parse_args()


def lib_repo():
    args = parse_args()

    if args.sub_command != 'add':
        exit(1)

    if not args.path:
        while True:
            path = input("Input the lib local path you want to upload: ")
            if path:
                args.path = path
                break
    if not args.name:
        while True:
            name = input("Input the lib name you want to upload: ")
            if name:
                args.name = name.strip()
                break
    if not args.platform:
        platform = input("Input the platform for the lib you want to upload: ")
        args.platform = platform
    if not args.arch:
        arch = input("Input the architecture for the lib you want to upload: ")
        args.arch = arch
    if not args.build:
        build = input("Input the build type for the lib you want to upload: ")
        args.build = build

    def strip(s):
        return s.strip() if s else None

    lib_data = LibData(name=args.name, version=strip(args.version),
                       platform=strip(args.platform),
                       arch=strip(args.arch),
                       build=strip(args.build))

    lib_repo = LibRepo().pull_repo()
    upload = True
    if lib_repo.exist(lib_data):
        while True:
            override = input("The lib you want to upload has existed in repo. Do you want to override that? yes/no")
            if override == 'yes':
                upload = True
                break
            elif override == 'no':
                upload = False
                break
            else:
                logger.info("Please input yes or no")

    if upload:
        update_lib(args.path, lib_data)


if __name__ == "__main__":
    lib_repo()





import io

import oss2
import os
import json
import zipfile
import shutil
import sys
from dataclasses import dataclass

OSS_BASE_PATH = 'arcsite-x-libs-repo'

@dataclass
class BucketInfo:
    endpoint: str
    name: str
    access_key_id: str
    access_key_secret: str
    

BucketInfo.endpoint = os.getenv('file_bucket_endpoint')
BucketInfo.name = os.getenv('file_bucket_name')
BucketInfo.access_key_id = os.getenv('file_bucket_access_key_id')
BucketInfo.access_key_secret = os.getenv('file_bucket_access_key_secret')


BUCKET_OBJECT = None


def get_bucket() -> oss2.Bucket:
    global BUCKET_OBJECT

    if BUCKET_OBJECT:
        return BUCKET_OBJECT

    if not BucketInfo.endpoint or not BucketInfo.name or not BucketInfo.access_key_id or not BucketInfo.access_key_secret:
        raise Exception('File bucket info not complete.')

    auth = oss2.Auth(BucketInfo.access_key_id, BucketInfo.access_key_secret)
    BUCKET_OBJECT = oss2.Bucket(auth, BucketInfo.endpoint, BucketInfo.name)
    return BUCKET_OBJECT


def zip_dir(files_dir, dest_file):
    with zipfile.ZipFile(dest_file, 'w', zipfile.ZIP_DEFLATED, True) as ziph:
        _, zip_name = os.path.split(dest_file)
        for root, dirs, files in os.walk(files_dir):
            for f in files:
                fu = f
                if fu.startswith('.') or os.path.join(root, fu) == dest_file:
                    continue
                p = os.path.join(root, fu)
                rel = os.path.relpath(os.path.join(root, fu), files_dir)
                ziph.write(p, rel)
                

def unzip_file(zip_file, dest_dir):
    with zipfile.ZipFile(zip_file, 'r', zipfile.ZIP_DEFLATED, True) as z:
        z.extractall(dest_dir)


def upload_file(file_path, file_key):
    with open(file_path, 'rb') as f:
        get_bucket().put_object(file_key, f)


def download_file(file_key, to_path):
    get_bucket().get_object_to_file(file_key, to_path)


def get_file_obj_2_str(file_key, not_exist_exception=False):
    try:
        result = get_bucket().get_object(file_key).read()
        return result
    except oss2.exceptions.NoSuchKey as e:
        if not_exist_exception:
            raise e
        else:
            return None


def upload_file_obj(file_key: str, file_obj):
    get_bucket().put_object(file_key, file_obj)


def is_legal_local_lib_dir(local_path: str, platform: str):
    p = os.path.join(local_path, platform)
    return os.path.exists(p) and os.path.isdir(p)


def get_lib_zip_file_name(name: str, version: str, platform: str):
    return platform + '.zip'


def get_lib_zip_file_key(name: str, version: str, platform: str):
    zip_file_name = get_lib_zip_file_name(name, version, platform)
    file_key = os.path.join(OSS_BASE_PATH, name, version, zip_file_name)
    file_key = file_key.replace('\\', '/')

    return file_key


def get_platform_name() -> str:
    p = sys.platform
    if p.startswith('linux'):
        return 'linux'
    if p.startswith('win32'):
        return 'windows'
    if p.startswith('darwin'):
        return 'macos'


def available_platforms():
    return ['windows', 'macos', 'linux', 'ios', 'android','ios_simulator']


def available_arches(platform: str):
    data = {
        "windows": ['x86', 'x64'],
        'macos': ['x86_64', 'arm64', 'x64'],
        'linux': [],
        'ios': [],
        'andriod': []
    }

    return data.get(platform, None)


def all_available_arches():
    all_arches = []
    for p in available_platforms():
        arches = available_arches(p)
        if arches:
            all_arches.extend(arches)

    return all_arches


def available_build_names():
    return ['release', 'debug', 'Release', 'Debug']



from setuptools import setup, find_packages

setup(
    name='lib_repo_tool',
    version='1.0.1',
    author='Andy Zhuang',
    author_email='xiaolong.zhuang@gmail.com',
    description='Put depended libs to oss file bucket',
    install_requires=['oss2==2.14.0', 'dacite==1.6.0', 'tabulate'],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'lib-get=repo_tool:lib_get',
            'lib-repo=repo_tool:lib_repo'
        ]
    }
)